# Collect data with python from smartis.re
# Mickaël Mézino - CIRAD - Février - 2021
#!/usr/bin/python3
import os 
import requests
import base64
import datetime


# Fonction de validation de l'identité
# Argument 1 : login utilisé pour se connecter à smartis.re
# Argument 2 : mot de passe crypté 
# Renvoi l'entete validée contenant l'authentification basic encodé en base64
def get_credential(username,passwd):
    # Encodage de l'entete pour la requete https :
    credentials=(username + ':' + passwd).encode('utf-8')
    base64_encoded_credentials = base64.b64encode(credentials).decode('utf-8')
    headers = {
        'Authorization': 'Basic ' + base64_encoded_credentials
    }
    url = 'https://smartis.re/api/WSMeteo?long=55.307965278625495&lat=-21.140868798573788&startdate=2018-01-01&enddate=2018-01-02&format=raw'
    responses = requests.get(url,headers = headers)
    if responses.status_code==200:
        return headers
    else:
        return -1



# Fonction de récupération de données météo en un point situé à (longitude,latitude).
# Arg 1 : credential => obtenu par appel de la fonction get_credential
# Arg 2 : start_date => Date de début de la série météo souhaitée
# Arg 3 : end_date => Date de fin de la série météo souhaitée
# Arg 4 : latitude => latitude de lieu ou la série est souhaitée en ° décimaux
# Arg 5 : longitude => latitude de lieu ou la série est souhaitée en ° décimaux
# Renvoi une série de donnée incluant Pluie,Rayonnement,Température mini, température maxi, température moyenne et Etp journalière.
def ws_geo_meteo(credential,start_date,end_date,latitude,longitude):
    url = 'https://smartis.re/api/WSMeteo?long='+longitude+'&lat='+latitude+'&startdate='+start_date+'&enddate='+end_date+'&format=raw'
    return ws_response(url,credential)



# Fonction de récupération de données météo en pied de la station indiquée par id_station.
# Arg 1 : credential => obtenu par appel de la fonction get_credential
# Arg 2 : id_station => numéro Insee de la station 
# Arg 3 : start_date => Date de début de la série météo souhaitée
# Arg 4 : end_date => Date de fin de la série météo souhaitée
# Arg 5 : presence_header => valeur 1 ou 0. 1 => renvoi l'entete avec les données, 0 => renvoi uniquement les données
# Renvoi une série de donnée sous forme CNA
def ws_mf_meteo(credential,id_station,start_date,end_date,presence_header):
    url = 'https://smartis.re/api/GET_CLIM_DATA_CIRAD?idstation='+id_station+'&startdate='+start_date+'&enddate='+end_date+'&haeder='+presence_header+'&format=raw'
    return ws_response(url,credential)


# Fonction de récupération de données météo en pied de la station indiquée par id_station.
# Arg 1 : credential => obtenu par appel de la fonction get_credential
# Arg 2 : id_station => numéro Insee de la station 
# Arg 3 : start_date => Date de début de la série météo souhaitée
# Arg 4 : end_date => Date de fin de la série météo souhaitée
# Arg 5 : parametre => valeur possible rr,glot,tn,tx,tm,etp,vt. il est possible de demander plusieurs paramètre en les séparant avec un | exemple : rr|glot|tm renverra pluie, rayonnement et température moyenne. 
# Arg 6 : frequence => indique la fréquence de sortie : H : données horaires, Q : données quotidienne
# Renvoi une série de donnée sous forme CNA
def ws_station_meteo(credential,id_station,start_date,end_date,parametre,frequence):
    url = 'https://smartis.re/api/GET_STATION_DATA_CIRAD?freq='+frequence+'&idstation='+id_station+'&startdate='+start_date+'&enddate='+end_date+'&params='+parametre+'&format=raw'
    return ws_response(url,credential)

# Fonction de récupération de la réserve utile en un point situé à (longitude,latitude).
# Arg 1 : credential => obtenu par appel de la fonction get_credential
# Arg 2 : latitude => latitude de lieu ou la série est souhaitée en ° décimaux
# Arg 3 : longitude => longitude de lieu ou la série est souhaitée en ° décimaux
# Renvoi la valeur de la réserve utile à la position demandé. 
def ws_ru(credential,latitude,longitude):
    url = 'https://smartis.re/api/WSRU?lat='+latitude+'&long='+longitude+'&format=raw'
    return ws_response(url,credential)

# Fonction de récupération de l'altitude en un point situé à (longitude,latitude).
# Arg 1 : credential => obtenu par appel de la fonction get_credential
# Arg 2 : latitude => latitude de lieu ou la série est souhaitée en ° décimaux
# Arg 3 : longitude => longitude de lieu ou la série est souhaitée en ° décimaux
# Arg 4 : unite => Unité soit degrée décimaux indiquez 'dd' ou mètre indiquer 'm' (coordonnée EPSG:32740)
# Renvoi la valeur de la réserve utile à la position demandé. 
def ws_ru(credential,latitude,longitude, unite='dd'):
    url = 'https://smartis.re/api/WSALTI?lat='+latitude+'&long='+longitude+'&unit='+unite+'&format=raw'
    return ws_response(url,credential)



# Fonction de récupération de données météo du projet BRIO.
# Arg 1 : credential => obtenu par appel de la fonction get_credential
# Arg 2 : latitude => latitude de lieu ou la série est souhaitée en ° décimaux
# Arg 3 : longitude => longitude de lieu ou la série est souhaitée en ° décimaux
# Arg 4 : start_date => Date de début de la série météo souhaitée. Les dates sont valides entre 2015 et 2100
# Arg 5 : end_date => Date de fin de la série météo souhaitée. Les dates sont valides entre 2015 et 2100
# Arg 6 : scenario => indique le code scénario de changement climatique souhaité. 3 scénarios existent : 
#  - SSP 1 avec réchauffement limité à 2°C => code scenario= ssp126
#  - SSP 2 avec réchauffement limité à 3°C => code scenario = ssp245 
#  - SSP 5 avec réchauffement atteignant les 6°C. => code scenario = ssp585
#
# Renvoi une série de donnée 
def ws_brio_data(credential,latitude,longitude,start_date,end_date,scenario):
    url = 'https://smartis.re/api/GET_CLIMATE_CHANGE_DATA?long='+longitude+'&lat='+latitude+'&startdate='+start_date+'&enddate='+end_date+'&scenario='+scenario+'&format=raw'
    return ws_response(url,credential)


# Renvoi une série de donnée 
def ws_data_grid(credential,latitude,longitude,start_date,end_date,scenario = '0'):
    url = 'https://smartis.re/api/GET_CLIMATE_GRID?long='+longitude+'&lat='+latitude+'&startdate='+start_date+'&enddate='+end_date+'&scenario='+scenario+'&format=raw'
    return ws_response(url,credential)


# Fonction de récupération d'une analyse fréquentielle journalière à une position données.
# Arg 1 : credential => obtenu par appel de la fonction get_credential
# Arg 2 : latitude => latitude du lieu ou la série est souhaitée en ° décimaux
# Arg 3 : longitude => longitude du lieu ou la série est souhaitée en ° décimaux
# Arg 4 : altitude => indique l'altitude du lieu souhaité en m par rapport au niveau de la mer. 
# Arg 5 : start_date => Date de début de la série météo souhaitée. Les dates sont valides avant la date courante.
# Arg 6 : end_date => Date de fin de la série météo souhaitée. Les dates sont valides avant la date courante.

# Renvoi une analyse fréquentielle en année médiane des données pluie, température, évapotranspiration, rayonnement pour chaque jour de l'année. 
# Le webservice calculs pour chaque jour de l'année la valeur médiane du paramètre souhaité (rr,tm,etp,glot) en prenant en compte 10 ans de données. 
# médiane (des valeurs du jour 1 de l'année 1, jour 1 de l'année 2,...., jour 1 année 10).
# médiane (des valeurs du jour 2 de l'année 1, jour 2 de l'année 2,...., jour 2 année 10).
# médiane (........................., .........................,.........................).
# médiane (des valeurs du jour n de l'année 1, jour n de l'année 2,...., jour n année 10).
# médiane (des valeurs du jour 365 de l'année 1, jour 365 de l'année 2,...., jour 365 année 10).
# les années de référence dépendent de l'année de début demandé. 
# Période =  [ Année début - 11 ; Année début - 1 ] pour avoir 10 années complète. 
def ws_meteo_frequency(credential,latitude,longitude,altitude,start_date,end_date):
    url = 'https://smartis.re/api/WSMeteoFrequencyAnalysis?long='+longitude+'&lat='+latitude+'&alt='+altitude+'&startdate='+start_date+'&enddate='+end_date+'&format=raw'
    return ws_response(url,credential)



# Fonction d'appel au modèle de croissance de la canne à sucre MOSICAS
# Arg 1 : credential => obtenu par appel de la fonction get_credential
# Arg 2 : latitude => latitude du lieu ou la série est souhaitée en ° décimaux
# Arg 3 : longitude => longitude du lieu ou la série est souhaitée en ° décimaux
# Arg 4 : start_date => Date de début de la série météo souhaitée. Les dates sont valides entre 01/01/2002 et end_date - 1
# Arg 5 : end_date => Date de fin de la série météo souhaitée. Les dates sont valides entre 01/01/2002 et jour courant - 1
# Arg 6 : remplissage => Taux de remplissage de la réserve utile en %
# Arg 7 : p0 => p-Factor fixé par défaut à 0.5 qui décrit la répartition entre la partie de la réserve où l'eau est facile d'accès et l'autre partie ou l'eau est difficile d'accès sans unité
# Arg 8 : ru => Réserve utile en mm 
# Arg 9 : profrac => Profondeur d'enracinement en m
# Arg 10 : ressurf => réservoir de surface pour en prendre en compte le mulch en mm
# Arg 11 : variete => Code variete sans unité
# Arg 12 : stade =>  code indiquant la nature => 1 repousse , 0 vierge
# Arg 13 : irdose => dose d'irrigation en mm
# Arg 14 : freqdose=> fréquence ou la dose d'irrigation sera appliquée en JOUR entier 
# Arg 15 : freq_sortie=> indique la fréquence ou les valeurs calculés par le modèle seront sortie vers l'écran ou le fichier. Si freq_sortie > durée du cycle, la dernière ligne de valeurs du modèle est sortie
# Retourne l'ensemble des variables calculées par le modèle MOSICAS (météo, bilan hydrique, croissance de la plante en cm, rendement) variable yldcan en t/ha
def ws_mosicas(credential,latitude,longitude,start_date,end_date,remplissage,p0,ru,profrac,ressurf,variete,stade,irdose,freqdose,freq_sortie,dose_is_var='0',tx_rempl='80'):
    url = 'https://smartis.re/api/WSMosicas?lat='+latitude+'&long='+longitude+'&startdate='+start_date+'&enddate='+end_date+'&remplissage='+remplissage+'&p0='+p0+'&ru='+ru+'&profrac='+profrac+'&ressurf='+ressurf+'&variete='+variete+'&stade='+stade+'&irdose='+irdose+'&freqdose='+freqdose+'&freq='+freq_sortie+'&isFixe='+dose_is_var+'&txRempl='+tx_rempl+'&format=raw'
    return ws_response(url,credential)

def ws_mosicas_grid(credential,latitude,longitude,start_date,end_date,remplissage,p0,ru,profrac,ressurf,variete,stade,irdose,freqdose,freq_sortie,code_scen='0',dose_is_var='0',tx_rempl='80'):
    if code_scen=='0':
        url = 'https://smartis.re/api/WSMosicas?lat='+latitude+'&long='+longitude+'&startdate='+start_date+'&enddate='+end_date+'&remplissage='+remplissage+'&p0='+p0+'&ru='+ru+'&profrac='+profrac+'&ressurf='+ressurf+'&variete='+variete+'&stade='+stade+'&irdose='+irdose+'&freqdose='+freqdose+'&freq='+freq_sortie+'&isFixe='+dose_is_var+'&txRempl='+tx_rempl+'&grid=1&format=raw'
    else:
        url = 'https://smartis.re/api/WSMosicas?lat='+latitude+'&long='+longitude+'&startdate='+start_date+'&enddate='+end_date+'&remplissage='+remplissage+'&p0='+p0+'&ru='+ru+'&profrac='+profrac+'&ressurf='+ressurf+'&variete='+variete+'&stade='+stade+'&irdose='+irdose+'&freqdose='+freqdose+'&freq='+freq_sortie+'&isFixe='+dose_is_var+'&txRempl='+tx_rempl+'&grid=1'+'&scenario='+code_scen+'&format=raw'
    return ws_response(url,credential)

# Fonction de transformation de la réponse des webservice. 
# la fonction transforme les données en matrice.
def get_clim_To_Ws(data):
    ldata = data.split('\n')
    tab = []
    # decodage des données : 
    n = len(ldata)
    for i in range(n-1):
        ligne = ldata[i].split(';')
        nligne = ligne
        col = len(ligne)   
        if i>0:
            for c in range(col):
                nligne[c] = str_float_choose(ligne[c])
        tab.append(nligne)
    return tab

# Renvoi la description du scénario correspondant au code:
def choix_scenario(code_scenario):
    switcher  = {
        'ssp126' : 'SSP 1 avec réchauffement limité à 2°C. Le scénario SSP1 (faible défi d’adaptation, faible défi d’atténuation), décrit un monde marqué par une forte coopération internationale, donnant la priorité au développement durable - 2015 - 2100',
        'ssp245' : 'SSP 2 avec réchauffement limité à 3°C. Le scénario SSP2 (défi d’adaptation moyen, défi d’atténuation moyen), décrit un monde caractérisé par la poursuite des tendances actuelles - 2015 - 2100',
        'ssp585' : 'SSP 5 avec réchauffement atteignant les 6°C. Le scénario SSP5 (faible défi d’adaptation, défi d’atténuation élevé) décrit un monde qui se concentre sur un développement traditionnel et rapide des pays en voie de développement, fondé sur une forte consommation d’énergie et des technologies émettrices de carbone; la hausse du niveau de vie permettrait d’augmenter la capacité d’adaptation, notamment grâce au recul de l’extrême pauvreté. 2015 - 2100',
        'hist'   : 'Donnée historique de 01/01/1981 - 31/12/2015',
        'ref'    : 'Données de référence ERA5 corrigé du biais par AROME (2.5 km); interpollé sur le une grille de 3x3 km - 2000 - 2019' 
    }
    return switcher.get(code_scenario,'Code scénario invalide')


# Renvoi le code du scénario correspondant au code en base postgresql :
def code_scenario(code_scenario):
    switcher  = {
        'ssp126' : '1',
        'ssp245' : '2',
        'ssp585' : '3',
        'hist'   : '4',
        'ref'    : '0'
    }
    return switcher.get(code_scenario,'Code scénario invalide')


# cette fonction vérifie le type possible d'une valeur. 
# si elle peut être converti dans un format numérique, elle converti sinon elle est renvoyé en chaine de caractère. 
def str_float_choose(value):
    try:
        nv = float(value)
    except ValueError:
        return str(value)
    if isinstance(nv,float):
        if (nv - int(nv))==0:
            return int(nv) # si c'est un float mais que la différence avec l'entier le plus proche est 0, cela signifie que le floatant peut etre renvoyé sous la forme entière (int).
        else:
            return nv


def choix_variete(variete):
    list_variete = {
        'B 5992': 1,
        'B 69379': 2,
        'B 69566': 3,
        'B 8008': 4,
        'B 80689': 5,
        'B 82139': 6,
        'B 47528': 7,
        'B 51129': 8,
        'CO 6415': 9,
        'R570': 10,
        'R579': 11,
        'R580': 12,
        'R581': 13,
        'R582': 14,
        'R583': 15,
        'NCO 376': 18,
        'Mex 68/200': 19,
        'R577': 20,
        'R92 804': 21,
        'R585': 22
    }
    return str(list_variete.get(variete,0))


def ws_response(url,credential):
    responses = requests.get(url,headers = credential)
    if responses.status_code==200:
        data = responses.content.decode("utf-8")
        return get_clim_To_Ws(data)
    else:
        print('Status: connexion fail. Check your access credentials ')